/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.core.authentication;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.bindings.*;
import net.sf.jguard.core.authentication.callbacks.AuthenticationSchemeHandlerCallback;
import net.sf.jguard.core.authentication.callbacks.InetAddressCallback;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;
import net.sf.jguard.core.authentication.schemes.HookFormSchemeHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.security.auth.Subject;
import javax.security.auth.callback.*;
import javax.security.auth.login.LoginException;
import java.security.Permission;
import java.util.*;


/**
 * Authenticate a user in an application with a specific {@link AuthenticationBindings}.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @since 1.1
 */
public class AuthenticationServicePoint {
    private static final String PASSWORD = "password";
    private static final String LOGIN = "login";


    private static final Logger logger = LoggerFactory.getLogger(AuthenticationServicePoint.class.getName());


    /**
     * return <b>true</b> if the user <b>tries</b> to answer to an authentication challenge.
     *
     * @param context
     * @param factory
     * @return
     */
    public static boolean answerToChallenge(AccessContext context, AuthenticationBindingsFactory factory) {
        List<AuthenticationSchemeHandler> authSchemeHandlers = factory.getAuthenticationSchemeHandlers();
        for (AuthenticationSchemeHandler handler : authSchemeHandlers) {
            boolean answerToChallenge = handler.answerToChallenge(context);
            if (answerToChallenge) {
                return true;
            }
        }

        return false;
    }

    /**
     * authenticate the user.
     * if the {@link AuthenticationBindings} implementation implements the
     * StatefulAuthenticationBindings interface, this method removes a possible old {@link AuthenticationUtils} object,
     * invalidate the session, create a new one, and bound the new Authenticationutils to it.
     */
    public static AuthenticationUtils authenticate(AuthenticationBindings authenticationBindings, String applicationName, String authenticationScope) throws AuthenticationException {
        authenticationBindings.setRequestAttribute(CoreConstants.REGISTRATION_DONE, Boolean.FALSE);

        //we grab the CallbackHandler to communicate during authentication with the user
        CallbackHandler cbh = authenticationBindings.getCallbackHandler();
        AuthenticationUtils authNUtils = null;
        try {
            //we grab the wrapper object which link the user via its session with authentication
            authNUtils = authenticationBindings.getAuthenticationUtils();

            //we use the wrapper object bound to user with the dedicated object(callabckHandler)
            //to communicate with him to authenticate
            authNUtils.login(applicationName, cbh);

            //for security reasons(man-in-the-middle attack for sessionID cookie),
            //we remove the old session if authentication is successfull
            //and create a new session

            if (authenticationBindings instanceof StatefulAuthenticationBindings) {
                Permission lastAccessDeniedPermission = null;
                //we remove this variable before invalidate the session
                //to prevent the session listener to erase the subject
                ((StatefulAuthenticationBindings) authenticationBindings).removeSessionAttribute(CoreConstants.AUTHN_UTILS);
                lastAccessDeniedPermission = (Permission) ((StatefulAuthenticationBindings) authenticationBindings).getSessionAttribute(CoreConstants.LAST_ACCESS_DENIED_PERMISSION);
                ((StatefulAuthenticationBindings) authenticationBindings).invalidateSession();

                ((StatefulAuthenticationBindings) authenticationBindings).setSessionAttribute(CoreConstants.AUTHN_UTILS, authNUtils);
                ((StatefulAuthenticationBindings) authenticationBindings).setSessionAttribute(CoreConstants.LAST_ACCESS_DENIED_PERMISSION, lastAccessDeniedPermission);

            }
            AuthenticationSchemeHandler authSchemeHandler = getAuthenticationSchemeHandler(authNUtils.getSubject(), authenticationBindings.getAuthenticationBindingsFactory());
            authSchemeHandler.authenticationSucceed(authenticationBindings.getContext());
            authNUtils.setStatus(AuthenticationStatus.SUCCESS);
            return authNUtils;

        } catch (AuthenticationContinueException ace) {
            //the current AuthenticationScheme needs multiple roundtrips
            logger.debug("authentication is not yet complete. a new exchange between client and server is required " + ace.getMessage());
            authNUtils.setStatus(AuthenticationStatus.CONTINUE);
            return authNUtils;
        } catch (AuthenticationChallengeException ace) {
            //the callbackHandle handle this case. we only need here to go back the call to the user
            logger.debug("authentication challenge built. a new exchange between client and server is required " + ace.getMessage());
            authNUtils.setStatus(AuthenticationStatus.FAILURE);
            return authNUtils;

        } catch (LoginException e) {

            logger.debug("authentication failed " + e.getMessage(), e);
            String messageError = null;
            messageError = e.getLocalizedMessage();
            //we store in the user' session the reason the authentication failed
            authenticationBindings.setRequestAttribute(CoreConstants.LOGIN_EXCEPTION_MESSAGE, messageError);
            authenticationBindings.setRequestAttribute(CoreConstants.LOGIN_EXCEPTION_CLASS, e.getClass());
            AuthenticationSchemeHandler authSchemeHandler = (AuthenticationSchemeHandler) authenticationBindings.getRequestAttribute(CoreConstants.AUTHENTICATION_SCHEME_HANDLER);
            authSchemeHandler.authenticationFailed(authenticationBindings.getContext());
            authNUtils.setStatus(AuthenticationStatus.FAILURE);
            return authNUtils;

        }
    }

    /**
     * grab into the {@link Subject} the name of the AuthenticationSchemeHandler,
     * to select among ones registered into the {@link AuthenticationBindings} and return it.
     *
     * @param subject
     * @return
     */
    public static AuthenticationSchemeHandler getAuthenticationSchemeHandler(Subject subject, AuthenticationBindingsFactory authNBindingsFactory) {
        String authSchemeHandlerName = getAuthSchemeHandlerName(subject);
        if (authSchemeHandlerName == null) {
            throw new IllegalArgumentException(" Subject does not contains a JGuardCredential with a key='authSchemeHandlerName' and a value not null ");
        }
        List<AuthenticationSchemeHandler> authSchemeHandlers = authNBindingsFactory.getAuthenticationSchemeHandlers();
        AuthenticationSchemeHandler authSchemeHandler = null;
        for (AuthenticationSchemeHandler authHandler : authSchemeHandlers) {
            if (authSchemeHandlerName.equals(authHandler.getName())) {
                authSchemeHandler = authHandler;
                break;
            }
        }
        return authSchemeHandler;
    }

    /**
     * authSchemeHandler name is stored into the Subject credentials set.
     *
     * @param subject
     * @return
     */
    private static String getAuthSchemeHandlerName(Subject subject) {
        Set<JGuardCredential> credentials = subject.getPublicCredentials(JGuardCredential.class);
        for (JGuardCredential cred : credentials) {
            if (CoreConstants.AUTHENTICATION_SCHEME_HANDLER_NAME.equals(cred.getName())) {
                return (String) cred.getValue();
            }
        }
        return null;
    }

    /**
     * impersonate the current user with the provided callbacks.
     * note that the user present in the authenticationBindings does not change.
     * a wrapping mechanism for authenticationSchemeHandler and AuthenticationBindings impersonate
     * the user, but the underlying authenticationBindings contains the real user.
     *
     * @param authenticationBindings
     * @param authHandlers
     * @param applicationName
     * @param callbacks
     * @param authenticationScope
     * @return
     */
    public static AuthenticationUtils authenticateWithImpersonation(AuthenticationBindings authenticationBindings, List<AuthenticationSchemeHandler> authHandlers, String applicationName, List<Callback> callbacks, String authenticationScope) throws AuthenticationException {
        AuthenticationSchemeHandlerCallback cb = new AuthenticationSchemeHandlerCallback();
        cb.setAuthenticationSchemeHandlerName("HOOK");
        callbacks.add(cb);
        ImpersonationAuthenticationBindings impersonatedAuthenticationBindings = null;
        if (authenticationBindings instanceof StatefulAuthenticationBindings) {
            impersonatedAuthenticationBindings = new StatefulImpersonationAuthenticationBindings(authenticationBindings, callbacks);
        } else {
            impersonatedAuthenticationBindings = new ImpersonationAuthenticationBindings(authenticationBindings, callbacks);
        }
        impersonatedAuthenticationBindings.addAuthenticationSchemeHandlerToFactory(authHandlers);
        return AuthenticationServicePoint.authenticate(impersonatedAuthenticationBindings, applicationName, authenticationScope);
    }

    /**
     * impersonate the current user as a Guest user with the related credentials.
     * it set the NameCallback to <b>guest<b/>,the PasswordCallback to <b>guest</b>,
     * the InetAddressCallback host address and host name to 127.0.0.1 and localhost.
     * note that the user present in the authenticationBindings does not change.
     * a wrapping mechanism for authenticationSchemeHandler and AuthenticationBindings impersonate
     * the user as a guest, but the underlying authenticationBindings contains the real user.
     *
     * @param authenticationBindings underlying authenticationBindings which contains the <b>real</b>
     * @param applicationName
     * @param authenticationScope    user.
     * @return
     */
    public static AuthenticationUtils impersonateAsGuest(AuthenticationBindings authenticationBindings, String applicationName, String authenticationScope) throws AuthenticationException {

        List<Callback> callbacks = new ArrayList<Callback>(4);
        NameCallback nameCallback = new NameCallback(LOGIN);
        nameCallback.setName(CoreConstants.GUEST);
        callbacks.add(nameCallback);

        PasswordCallback pwdCbk = new PasswordCallback(PASSWORD, false);
        pwdCbk.setPassword(CoreConstants.GUEST.toCharArray());
        callbacks.add(pwdCbk);

        InetAddressCallback address = new InetAddressCallback();
        final String localIP = "127.0.0.1";
        address.setHostAdress(localIP);
        address.setHostName("localhost");
        callbacks.add(address);


        LanguageCallback languageCallback = new LanguageCallback();
        languageCallback.setLocale(Locale.getDefault());
        callbacks.add(languageCallback);

        //add HookFormSchemeHandler
        List<AuthenticationSchemeHandler> authenticationSchemeHandlers = new ArrayList<AuthenticationSchemeHandler>(1);
        Map<String, String> parameters = new HashMap<String, String>(2);
        authenticationSchemeHandlers.add(new HookFormSchemeHandler(parameters, authenticationBindings.getAuthenticationBindingsFactory()));
        return authenticateWithImpersonation(authenticationBindings, authenticationSchemeHandlers, applicationName, callbacks, authenticationScope);
    }


}
