/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.callbacks;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.Configuration;
import javax.security.auth.spi.LoginModule;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindings;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * the <i>keystone</i> of interactions between loginModules (the JAAS authentication process),
 * {@link AuthenticationBindings} implementation and its {@link AuthenticationSchemeHandler}s.
 * To support a new <i>communication technology</i>, you require to extends this class, and the related 
 * {@link AuthenticationSchemeHandler}s.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public abstract class AbstractCallbackHandler implements CallbackHandler{

    private static final Logger logger = LoggerFactory.getLogger(AbstractCallbackHandler.class.getName());
    protected AccessContext context = null;
    private List<AuthenticationSchemeHandler> authenticationSchemeHandlers = null;
    private AuthenticationBindings authBindings;

    
    public AbstractCallbackHandler(AuthenticationBindings authBindings,AccessContext context,List<AuthenticationSchemeHandler> authenticationSchemeHandlers){
        this.context = context;
        this.authBindings = authBindings;
        this.authenticationSchemeHandlers = authenticationSchemeHandlers;
        if(authenticationSchemeHandlers.size()==0){
            throw new IllegalArgumentException("no authenticationSchemeHandlers are registered ");
        }
    }
    
     private Collection<Class> getCallbacksTypes(List<Callback> cbks) {
        Set<Class> types = new HashSet<Class>();
        for(Callback cbk:cbks){
            types.add(cbk.getClass());
        }
        return types;
    }

    /**
     * choice of the Authentication Scheme is done by {@link LoginModule}s executed according to the {@link Configuration}.
     * Interactions between multiple loginmodules, to choose which authentication scheme must be encountered
     * among multiple ones, is possible. Each Authentication Scheme is implemented with a loginModule, and its related {@link AuthenticationSchemeHandler}
     * implementation. each {@link AuthenticationSchemeHandler}s are registered in the {@link AuthenticationBindings} implementation.
     * Correlation between a loginModule and an AuthentitcationSchemeHandler is expressed with specific callbacks.
     * this method implements this correlation.
     * @param authenticationSchemeHandlers
     * @param cbks
     * @return
     */
    private AuthenticationSchemeHandler getAuthenticationSchemeHandler(List<AuthenticationSchemeHandler> authenticationSchemeHandlers,List<Callback> cbks) {
        //callbacks required by the current LoginModule (which can contains callbacks not directly related to authenticationSchemeHandler)
        Collection<Class> requiredCallbackTypes = getCallbacksTypes(cbks);
        for(AuthenticationSchemeHandler authSchemeHandler:authenticationSchemeHandlers){
            //callbacks which can be filled by the current AuthenticationSchemeHandler
            Collection<Class> callbackTypes = authSchemeHandler.getCallbackTypes();
            //if all callbacks used by the AuthenticationSchemeHandler are present in the callbacks asked by loginmodule
            //we use this authenticationSchemehandler
            if(requiredCallbackTypes.containsAll(callbackTypes)){
                //callbacks types identify which authenticationSchemeHandler is required 
                //among multiple ones registered in the AuthenticationBindings and contained in the CallbackHandler
                return authSchemeHandler;
            }
            
        }
       
        return null;
    }
    
    public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
        List<Callback> cbks = Arrays.asList(callbacks);
        handleNonSchemeCallbacks(cbks);
        
        AuthenticationSchemeHandler authenticationSchemeHandler = getAuthenticationSchemeHandler(authenticationSchemeHandlers,cbks);
        authBindings.setRequestAttribute(CoreConstants.AUTHENTICATION_SCHEME_HANDLER, authenticationSchemeHandler);
        //handle method is called multiple times by loginmodules
        //some LoginModules does not put in place any authenticationSchemeHandler
        if(authenticationSchemeHandler==null){
            return;
        }
        
        //we set the AuthenticationSchemeHandlerCallback
        for(Callback callback : callbacks){
            if(callback instanceof AuthenticationSchemeHandlerCallback){
                AuthenticationSchemeHandlerCallback cbk = (AuthenticationSchemeHandlerCallback)callback;
                cbk.setAuthenticationSchemeHandlerName(authenticationSchemeHandler.getName());
                break;
            }
        }
        
        if(!authenticationSchemeHandler.answerToChallenge(authBindings.getContext())){
            //user has not yet tried to answer to an authentication challenge
            //we build a new challenge in response
            try {
                authenticationSchemeHandler.buildChallenge(authBindings.getContext());
            } catch (AuthenticationException ex) {
                logger.error(ex.getMessage());
                throw new RuntimeException(ex);
            }
            
            if(isAsynchronous()){
                throw new AsynchronousCallbackException(null,authenticationSchemeHandler.getName());
            }
        }

        authenticationSchemeHandler.handleSchemeCallbacks(authBindings.getContext(),cbks);
    }

    protected abstract void handleNonSchemeCallbacks(List<Callback> cbks)throws UnsupportedCallbackException;

    /**
     * define if the communication between client and server is non-blocking (return <b>true</b>) or blocking (return <b>false</b>).
     * @return
     */
    protected abstract boolean isAsynchronous();
}
