/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.configuration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.Configuration;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPointOptions;
import net.sf.jguard.core.util.XMLUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * utility class to deal with the {@link javax.security.auth.login.Configuration} class.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class ConfigurationHelper {
	 
	 private static String scope;
	 
	 
	 private static boolean configurationInstalled = false;
     private static Logger logger = LoggerFactory.getLogger(ConfigurationHelper.class.getName());
     private final static String COM_SUN_SECURITY_AUTH_LOGIN_CONFIG_FILE = "com.sun.security.auth.login.ConfigFile";

    /**
     * add the required AppConfigurationEntry when no none is configured for the webapp.
     * @param applicationName
     * @param authenticationSettings
     * @param debug
     */
    public static void addConfigurationEntryForApplication(JGuardConfiguration configuration,String applicationName,Map authenticationSettings,boolean debug){
        List webappEntries = buildAppConfigurationEntries(applicationName,authenticationSettings,debug);
        configuration.addConfigEntriesForApplication(applicationName,webappEntries);
    }


    /**
     * install JGuardConfiguration.
     * @param includeOldConfiguration include informations contained in the replaced configuration instance
     * @param includeConfigFromJavaParam
     * @throws RuntimeException
     */
	public static void installConfiguration(JGuardConfiguration jGuardConf,boolean includeConfigFromJavaParam) throws RuntimeException {
		if(!configurationInstalled){
			includeConfigFromJavaParam = false;
		}
			
		
        if(includeConfigFromJavaParam){
        	//TODO build a Configuration object to include from the configuration file
        	try {
				Class defaultConfigClass = Class.forName(ConfigurationHelper.COM_SUN_SECURITY_AUTH_LOGIN_CONFIG_FILE);
				Configuration defaultConfiguration = (Configuration) defaultConfigClass.newInstance();
	        	jGuardConf.includeConfiguration(defaultConfiguration);
			} catch (ClassNotFoundException e) {
				logger.error(ConfigurationHelper.COM_SUN_SECURITY_AUTH_LOGIN_CONFIG_FILE+" class cannot be found "+e.getMessage(),e);
			} catch (InstantiationException e) {
				logger.error(ConfigurationHelper.COM_SUN_SECURITY_AUTH_LOGIN_CONFIG_FILE+" class cannot be instantiated "+e.getMessage(),e);
			} catch (IllegalAccessException e) {
				logger.error(ConfigurationHelper.COM_SUN_SECURITY_AUTH_LOGIN_CONFIG_FILE+" class cannot be accessed "+e.getMessage(),e);
			}

        }

	}


    /**
     * we override the current JVM-scope {@link Configuration} instance, and include it into the
     * set Jguard's {@link Configuration} instance depending on the includeOldConfiguration flag.
     * @param includeOldConfiguration
     * @return
     * @throws RuntimeException
     */
	private static JGuardConfiguration installWithOldConfig(boolean includeOldConfiguration) throws RuntimeException {
		JGuardConfiguration jGuardConf;
		Configuration oldConfiguration = null;
        boolean skipOldConfig = false;
        try{
          oldConfiguration = Configuration.getConfiguration();
          logger.debug(" oldConfiguration="+oldConfiguration.getClass().getName());
          logger.debug(" oldConfiguration="+oldConfiguration);

        }catch(SecurityException sex){
          skipOldConfig = true;
          logger.debug(" addConfigurationEntryForWebapp() -  exception raised when we try to retrieve the default Configuration instance ");
          logger.debug(" addConfigurationEntryForWebapp() - "+ sex.getMessage());
          logger.debug(" jGuard will not include the old Configuration ");
        }catch(NullPointerException npe){
            skipOldConfig = true;
            logger.debug("addConfigurationEntryForWebapp() - a NullPointerException has been raised when the default configuration :no configuration is defined ");
            logger.debug("addConfigurationEntryForWebapp() - "+ npe.getMessage());
        }

        //we override the current Configuration to JGuardConfiguration
        //only if the current Configuration is not a JGuardConfiguration instance
        if(oldConfiguration== null ||
        		(!oldConfiguration.getClass().getName().equals(JGuardConfiguration.class.getName()))){
	          jGuardConf = new JGuardConfiguration();
	          Configuration.setConfiguration(jGuardConf);
	          logger.info(" JGuardConfiguration is set ");
	    //it is a JGuardConfiguration but with a class not loaded with this classloader
        }else if(!oldConfiguration.getClass().equals(JGuardConfiguration.class)
        		 && oldConfiguration.getClass().getName().equals(JGuardConfiguration.class.getName())){
        	logger.error(" jGuard_jvm must be placed under the shared libraries directory or on the jvm side, not in the WEB-INF/lib directory of the webapp ");
        	throw new RuntimeException(" jGuard_jvm must be placed under the shared libraries directory or on the jvm side, not in the WEB-INF/lib directory of the webapp ");
        }else{
          jGuardConf = (JGuardConfiguration) Configuration.getConfiguration();
          logger.debug("configuration="+oldConfiguration.getClass().getName());
          logger.debug(" JGuardConfiguration is already set ");
      	  // we don't include the old Configuration
      	  //because the old Configuration is kept: it's already a jGuardConfiguration
      	  skipOldConfig = true;
        }

        //there is no configuration entries for the webapp
        if(!skipOldConfig && includeOldConfiguration == true){
        	logger.info(" jGuard include the old Configuration ");
           jGuardConf.includeConfiguration(oldConfiguration);
        }

        //we should only execute one time this method per webapp
        configurationInstalled = true;
        return jGuardConf;
	}


    /**
     * build the list of AppConfigurationEntry.
     * @param applicationName
     * @param authSettings
     * @param debug
     * @return list built
     */
    public static List buildAppConfigurationEntries(String applicationName,Map authSettings,boolean debug) {
    	List appConfigurationEntryList = new ArrayList();

        List loginModules = (List)authSettings.get(CoreConstants.LOGIN_MODULES);
        if(loginModules==null){
        	throw new IllegalArgumentException(" no loginModules have been configured for the application="+applicationName);
        }
        Iterator itLoginModules = loginModules.iterator();
        while(itLoginModules.hasNext()){
               Map loginModuleMap = (Map)itLoginModules.next();
               String loginModuleClassName = (String)loginModuleMap.get(CoreConstants.NAME);
               String loginModuleFlag = (String)loginModuleMap.get(CoreConstants.FLAG);
               AppConfigurationEntry.LoginModuleControlFlag controlFlag;
               if(loginModuleFlag.equalsIgnoreCase(CoreConstants.REQUIRED)){
                   controlFlag = AppConfigurationEntry.LoginModuleControlFlag.REQUIRED;
               }else if(loginModuleFlag.equalsIgnoreCase(CoreConstants.OPTIONAL)){
                   controlFlag = AppConfigurationEntry.LoginModuleControlFlag.OPTIONAL;
               }else if(loginModuleFlag.equalsIgnoreCase(CoreConstants.REQUISITE)){
                   controlFlag = AppConfigurationEntry.LoginModuleControlFlag.REQUISITE;
               }else if(loginModuleFlag.equalsIgnoreCase(CoreConstants.SUFFICIENT)){
                   controlFlag = AppConfigurationEntry.LoginModuleControlFlag.SUFFICIENT;
               }else{
            	   throw new IllegalArgumentException(" invalid loginModuleControlFlag ="+loginModuleFlag+" is neither OPTIONAL,REQUIRED,REQUISITE nor SUFFICIENT ");
               }
               Map loginModuleOptions = (Map)loginModuleMap.get(CoreConstants.LOGIN_MODULE_OPTIONS);
               loginModuleOptions.put(PolicyEnforcementPointOptions.APPLICATION_NAME.getLabel(),applicationName);
               AppConfigurationEntry entry = new AppConfigurationEntry(loginModuleClassName,controlFlag,loginModuleOptions);
               appConfigurationEntryList.add(entry);
        }
        if(appConfigurationEntryList.size()==0){
        	throw new IllegalArgumentException(" no loginModules have been configured for the application="+applicationName);
        }
        return appConfigurationEntryList;
    }


	public static String getScope() {
		return scope;
	}


        /**
	 * load a configuration file from a location with an appHomePath prefix.
	 * @param configurationLocation
	 * @param appHomePath
	 * @return
	 */
	public static Map loadConfiguration(String configurationLocation, String appHomePath){

		Document doc = XMLUtils.read(configurationLocation);

		//authentication part
		Element authentication = doc.getRootElement().element(CoreConstants.AUTHENTICATION);
		Map authenticationMap = new HashMap();
		authenticationMap.put(CoreConstants.SCOPE, authentication.element(CoreConstants.SCOPE).getTextTrim());
		authenticationMap.put(CoreConstants.DEBUG, authentication.element(CoreConstants.DEBUG).getTextTrim());
		authenticationMap.put(CoreConstants.INCLUDE_OLD_CONFIG, authentication.element(CoreConstants.INCLUDE_OLD_CONFIG).getTextTrim());
		authenticationMap.put(CoreConstants.INCLUDE_CONFIG_FROM_JAVA_PARAM, authentication.element(CoreConstants.INCLUDE_CONFIG_FROM_JAVA_PARAM).getTextTrim());
		authenticationMap.put(CoreConstants.INCLUDE_POLICY_FROM_JAVA_PARAM, authentication.element(CoreConstants.INCLUDE_POLICY_FROM_JAVA_PARAM).getTextTrim());
		if(authentication.element(CoreConstants.DIGEST_ALGORITHM)!=null){
			authenticationMap.put(CoreConstants.DIGEST_ALGORITHM,authentication.element(CoreConstants.DIGEST_ALGORITHM).getTextTrim());
		}
		if(authentication.element(CoreConstants.SALT)!=null){
			authenticationMap.put(CoreConstants.SALT,authentication.element(CoreConstants.SALT).getTextTrim());
		}
		//loginModules configuration
		List loginModuleElementsList = authentication.element(CoreConstants.LOGIN_MODULES).elements(CoreConstants.LOGIN_MODULE);
		List loginModules = new ArrayList();
		Iterator itLoginModuleElementsList = loginModuleElementsList.iterator();
		while(itLoginModuleElementsList.hasNext()){
			Element loginModule = (Element)itLoginModuleElementsList.next();

			Map loginModuleMap = new HashMap();
			loginModuleMap.put(CoreConstants.NAME,loginModule.element(CoreConstants.NAME).getTextTrim());
			loginModuleMap.put(CoreConstants.FLAG,loginModule.element(CoreConstants.FLAG).getTextTrim());
			Element loginModuleOpts = loginModule.element(CoreConstants.LOGIN_MODULE_OPTIONS);
			if(loginModuleOpts!=null){
				List loginModuleOptsList = loginModuleOpts.elements(CoreConstants.OPTION);
				Iterator itLoginModuleOpts = loginModuleOptsList.iterator();
				Map loginModulesOptions = new HashMap();
				while (itLoginModuleOpts.hasNext()){
					Element option = (Element)itLoginModuleOpts.next();
					String name = option.element(CoreConstants.NAME).getTextTrim();
					String value = option.element(CoreConstants.VALUE_MARKUP).getTextTrim();
					if(CoreConstants.AUTHENTICATION_XML_FILE_LOCATION.equals(name)||CoreConstants.AUTHENTICATION_DATABASE_FILE_LOCATION.equals(name)){
						value=appHomePath+value;
					}
					loginModulesOptions.put(name,value);
				}
				loginModuleMap.put(CoreConstants.LOGIN_MODULE_OPTIONS, loginModulesOptions);
			}else{
				//there are no options for the loginmodule
				loginModuleMap.put(CoreConstants.LOGIN_MODULE_OPTIONS,new HashMap());
			}
			loginModules.add(loginModuleMap);
		}

		authenticationMap.put(CoreConstants.LOGIN_MODULES, loginModules);

		//authenticationManager configuration
		authenticationMap.put(CoreConstants.AUTHENTICATION_MANAGER, authentication.element(CoreConstants.AUTHENTICATION_MANAGER).getTextTrim());

		Map authenticationManagerOptions = new HashMap();
		Element authentManagerOptsElement = authentication.element(CoreConstants.AUTHENTICATION_MANAGER_OPTIONS);
		List authentManagerOptsList = authentManagerOptsElement.elements(CoreConstants.OPTION);
		Iterator authentManagerOpts = authentManagerOptsList.iterator();
		while (authentManagerOpts.hasNext()){
			Element option = (Element)authentManagerOpts.next();
			String name = option.element(CoreConstants.NAME).getTextTrim();
			String value = option.element(CoreConstants.VALUE_MARKUP).getTextTrim();
			if(CoreConstants.AUTHENTICATION_XML_FILE_LOCATION.equals(name)||CoreConstants.AUTHENTICATION_DATABASE_FILE_LOCATION.equals(name)){
     	    	value=appHomePath+value;
     	    }
			authenticationManagerOptions.put(name,value);
		}

		authenticationMap.put(CoreConstants.AUTHENTICATION_MANAGER_OPTIONS, authenticationManagerOptions);

		return authenticationMap;
	}
}
