/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authorization;

import net.sf.jguard.core.authorization.manager.AuthorizationManagerFactory;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.ehcache.CacheException;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPointOptions;
import net.sf.jguard.core.authorization.permissions.PermissionUtils;
import net.sf.jguard.core.authorization.policy.MultipleAppPolicy;

import net.sf.jguard.core.authorization.manager.AuthorizationManager;
import net.sf.jguard.core.util.XMLUtils;

import org.dom4j.Document;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * utility class used to load Authorization configuration
 * and to create and register an {@link AuthorizationManager}.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class AuthorizationHelper {

	private static final Logger logger = LoggerFactory.getLogger(AuthorizationHelper.class.getName());


	/**
	 * create an {@link AuthorizationManager} and register it into the {@link MultipleAppPolicy}.
	 * @param authorizationOptions
	 * @param applicationName
	 * @return AuthorizationManager created and registered
	 */
	public static AuthorizationManager initAuthorization(Map authorizationOptions,String applicationName){
		

		//permission caching section
		if ( !("true".equals(authorizationOptions.get(CoreConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING)))){
			PermissionUtils.setCachesEnabled(false);
		}else{
			// by default, permission resolution caching is activated
			try {
				PermissionUtils.createCaches();
				PermissionUtils.setCachesEnabled(true);
			} catch (CacheException e) {
				logger.warn("Failed to activate permission resolution caching : " + e.getMessage());
				PermissionUtils.setCachesEnabled(false);
			}
		}

		//authorizationManager creation
		AuthorizationManager am = null;
        Class clazz=null;
		try {
			// Create new authorization manager based on settings
			authorizationOptions.put(PolicyEnforcementPointOptions.APPLICATION_NAME.getLabel(),applicationName);
			if (logger.isDebugEnabled()) {
				logger.debug(" authorizationManagerOptions="+ authorizationOptions);
			}
			
			String authorizationManagerImplName =(String)authorizationOptions.get(CoreConstants.AUTHORIZATION_MANAGER);
            
			clazz = Thread.currentThread().getContextClassLoader().loadClass(authorizationManagerImplName);

			AuthorizationManagerFactory.createAuthorizationManager(clazz,authorizationOptions);
			am = AuthorizationManagerFactory.getAuthorizationManager();

		} catch (ClassNotFoundException e) {
			logger.error("ClassNotFoundException : "+e.getMessage());
			throw new IllegalArgumentException(e.getMessage()+"class="+clazz.getName());
		} catch (AuthorizationException e) {
			logger.error("AuthorizationException : "+e.getMessage());
            throw new IllegalArgumentException(e.getMessage()+"class="+clazz.getName());
		}

		return am;
	}

	/**
	 * loads the Authorization configuration file and returns settings.
	 * @param configurationLocation
	 * @param appHomePath
	 * @return map containing authorization settings
	 */
	public static Map loadConfiguration(String configurationLocation,String appHomePath){
		Map authorizationMap = new HashMap();
		Document doc = XMLUtils.read(configurationLocation);

		Element authorization = doc.getRootElement().element(CoreConstants.AUTHORIZATION);
		Element scope = authorization.element(CoreConstants.SCOPE);
		if (scope != null){
			authorizationMap.put(CoreConstants.SCOPE, scope.getTextTrim());
		}
		
		Element permissionResolutionCaching = authorization.element(CoreConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING);
		if (permissionResolutionCaching != null){
			authorizationMap.put(CoreConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING, permissionResolutionCaching.getTextTrim());
		}
		authorizationMap.put(CoreConstants.AUTHORIZATION_MANAGER,authorization.element(CoreConstants.AUTHORIZATION_MANAGER).getTextTrim());

		List authorizationList = authorization.element(CoreConstants.AUTHORIZATION_MANAGER_OPTIONS).elements(CoreConstants.OPTION);
		Iterator itAuthorizationOptions = authorizationList.iterator();
		while(itAuthorizationOptions.hasNext()){
			Element option = (Element)itAuthorizationOptions.next();
			String name = option.element(CoreConstants.NAME).getTextTrim();
			String value = option.element(CoreConstants.VALUE_MARKUP).getTextTrim();
			 if(CoreConstants.AUTHORIZATION_XML_FILE_LOCATION.equals(name)||CoreConstants.AUTHORIZATION_DATABASE_FILE_LOCATION.equals(name)){
	    	    	value=appHomePath+value;
	    	    }
			authorizationMap.put(name,value);
		}
		return authorizationMap;
	}

}
