/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.core.filters;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPointOptions;
import net.sf.jguard.core.authentication.*;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindings;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindingsFactory;
import net.sf.jguard.core.authentication.bindings.StatefulAuthenticationBindings;
import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;
import net.sf.jguard.core.authentication.schemes.StatefulAuthenticationSchemeHandler;
import net.sf.jguard.core.authorization.AuthorizationBindings;
import net.sf.jguard.core.authorization.PolicyDecisionPoint;
import net.sf.jguard.core.authorization.policy.AccessControllerUtils;
import net.sf.jguard.core.provisioning.ProvisioningServicePoint;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.security.auth.Subject;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Permissions;
import java.security.PrivilegedExceptionAction;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Map;

/**
 * {@link Filter} implementation dedicated to secure call in a safe way on multiple fields:
 * authentication of the call, authorization enforcement, check integrity and confidentiality.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class PolicyEnforcementPointFilter implements Filter {

    private AuthenticationBindingsFactory authenticationBindingsFactory = null;
    private PolicyDecisionPoint policyDecisionPoint = null;
    private ProvisioningServicePoint provisioningServicePoint = null;
    private static final Logger logger = LoggerFactory.getLogger(PolicyEnforcementPointFilter.class.getName());

    private String authenticationScope;
    private String applicationName;
    private boolean propagateThrowable;

    public static final String REDIRECT = "redirect";

    public PolicyEnforcementPointFilter(String authenticationBindingsFactoryImpl, Map<PolicyEnforcementPointOptions, String> options) {

        String filterConfigurationLocation = options.get(PolicyEnforcementPointOptions.CONFIGURATION_LOCATION);
        authenticationScope = options.get(PolicyEnforcementPointOptions.AUTHENTICATION_SCOPE);
        applicationName = options.get(PolicyEnforcementPointOptions.APPLICATION_NAME);
        String authorizationBindingsImpl = options.get(PolicyEnforcementPointOptions.AUTHORIZATION_BINDINGS);
        //initialize PolicyDecisionPoint
        this.policyDecisionPoint = initPolicyDecisionPoint(authorizationBindingsImpl);
        logger.debug("authenticationScope=" + authenticationScope);
        if (authenticationScope == null || "".equals(authenticationScope)) {
            throw new IllegalArgumentException("authenticationScope is null or empty");
        }

        //initialize authentication Bindings
        this.authenticationBindingsFactory = initAuthenticationBindings(authenticationBindingsFactoryImpl, filterConfigurationLocation);


        //initialize Provisioning Service Point
        String provisioningServicePointImpl = options.get(PolicyEnforcementPointOptions.PROVISIONING_SERVICE_POINT);
        if (provisioningServicePointImpl == null || "".equals(provisioningServicePointImpl)) {
            logger.info("provisioningServicePoint is not set ");
            /* }else{
                this.provisioningServicePoint = initProvisioningServicePoint(provisioningServicePointImpl, filterConfigurationLocation);
                policyDecisionPoint.addAlwaysGrantedPermissionsToPolicy(provisioningServicePoint.getGrantedPermissions());
            */
        }


        //include permissiosn grnated by some AuthenitcationSchemeHandler registered in AUthenticationBindings.
        policyDecisionPoint.addAlwaysGrantedPermissionsToPolicy(getGrantedPermissions(authenticationBindingsFactory));

        String pt = options.get(PolicyEnforcementPointOptions.PROPAGATE_THROWABLE);
        if (pt != null && !("").equals(pt)) {
            propagateThrowable = Boolean.parseBoolean(pt);
        }

    }

    private void authorize(AuthenticationBindings authenticationBindings, Subject subject, FilterChain chain) {
        //'decorate' context with authentication informations if needed
        // just before authorization phase
        authenticationBindings.process();

        boolean redirect = Boolean.valueOf((String) authenticationBindings.getRequestAttribute(PolicyEnforcementPointFilter.REDIRECT));
        if (!redirect) {
            propagateWithSecurity(authenticationBindings.getContext(), subject, chain);
        }
    }


    /**
     * return the {@link Subject} bound to the AccessContext.
     * @param authenticationBindings
     * @return current Subject
     */
    private Subject getCurrentSubject(AuthenticationBindings authenticationBindings) {
        AuthenticationUtils authNUtils = authenticationBindings.getAuthenticationUtils();
        return authNUtils.getSubject();

    }

    /**
     * return permissions granted according to {@link AuthenticationSchemeHandler} registered in the
     * AuthenticationBindings. Note that some permissions will be granted from some AuthenticationScheme
     * although if configuraiton of loginmodules does not implies the related AuthenticationSchemeHandler
     * present in the CallbackHandler.
     *
     * @param authenticationBindingsFactory
     * @return
     */
    private Permissions getGrantedPermissions(AuthenticationBindingsFactory authenticationBindingsFactory) {

        Permissions alwaysGrantedPermissions = new Permissions();
        //alwaysGrantedPermissions.add(authenticationBindings.getAuthenticationFailedPermission());

        Collection<AuthenticationSchemeHandler> authHandlers = authenticationBindingsFactory.getAuthenticationSchemeHandlers();
        for (AuthenticationSchemeHandler authHandler : authHandlers) {
            PermissionCollection permColl = authHandler.getGrantedPermissions();
            Enumeration<Permission> perms = permColl.elements();
            while (perms.hasMoreElements()) {
                alwaysGrantedPermissions.add(perms.nextElement());
            }
        }
        return alwaysGrantedPermissions;
    }

    public PolicyDecisionPoint getPolicyDecisionPoint() {
        return policyDecisionPoint;
    }

    public ProvisioningServicePoint getProvisioningServicePoint() {
        return provisioningServicePoint;
    }

    /**
     * return a Subject identified with the guest role.
     * it is a special user which represents any uanuthenticated users, with their access rights.
     * @param authenticationBindings
     * @return
     * @throws net.sf.jguard.core.authentication.AuthenticationException
      */
    private Subject getGuestSubject(AuthenticationBindings authenticationBindings) throws AuthenticationException {
        Subject localGuest;
        AuthenticationUtils authnUtils = AuthenticationServicePoint.impersonateAsGuest(authenticationBindings, applicationName, authenticationScope);
        if (authnUtils == null) {
            throw new IllegalStateException(" authenticationUtils is null in the Authenticationbindings");
        }
        if (AuthenticationStatus.SUCCESS.equals(authnUtils.getStatus())) {
            localGuest = authnUtils.getSubject();
        } else {
            throw new AuthenticationException("authentication of the guest user does not return a SUCCESS authentication status but " + authnUtils.getStatus().toString());
        }

        return localGuest;

    }

    /**
     * instantiate a PolicyDecisionPoint.
     *
     * @param authorizationBindingsImpl
     * @return instantiated PolicyDecisionPoint
     */
    private PolicyDecisionPoint initPolicyDecisionPoint(String authorizationBindingsImpl) {
        logger.debug("initializing PolicyDecisionPoint");
        logger.debug("authorizationBindingsImpl=" + authorizationBindingsImpl);
        AuthorizationBindings authorizationBindings;
        try {
            authorizationBindings = (AuthorizationBindings) Thread.currentThread().getContextClassLoader().loadClass(authorizationBindingsImpl).newInstance();

        } catch (InstantiationException iex) {
            logger.error(iex.getMessage(), iex);
            throw new IllegalArgumentException(iex);
        } catch (ClassNotFoundException cne) {
            logger.error(cne.getMessage(), cne);
            throw new IllegalArgumentException(cne);
        } catch (IllegalAccessException iae) {
            logger.error(iae.getMessage(), iae);
            throw new IllegalArgumentException(iae);
        }

        return new PolicyDecisionPoint(authorizationBindings);
    }

    private ProvisioningServicePoint initProvisioningServicePoint(String provisioningServicePointImpl, String filterConfigurationLocation) {
        logger.debug("initializing ProvisioningServicePoint");
        logger.debug("provisioningServicePointImpl=" + provisioningServicePointImpl);
        ProvisioningServicePoint psp;
        try {
            psp = (ProvisioningServicePoint) Thread.currentThread().getContextClassLoader().loadClass(provisioningServicePointImpl).newInstance();
            psp.init(filterConfigurationLocation);
        } catch (InstantiationException iex) {
            logger.error(iex.getMessage(), iex);
            throw new IllegalArgumentException(iex);
        } catch (ClassNotFoundException cne) {
            logger.error(cne.getMessage(), cne);
            throw new IllegalArgumentException(cne);
        } catch (IllegalAccessException iae) {
            logger.error(iae.getMessage(), iae);
            throw new IllegalArgumentException(iae);
        }

        return psp;
    }

    public AuthenticationBindingsFactory initAuthenticationBindings(String authenticationBindingsFactoryImpl, String filterConfigurationLocation) {
        if (authenticationBindingsFactoryImpl == null || "".equals(authenticationBindingsFactoryImpl)) {
            throw new IllegalArgumentException("authenticationBindingsImpl is null or empty");
        }
        if (filterConfigurationLocation == null || "".equals(filterConfigurationLocation)) {
            throw new IllegalArgumentException("filterConfigurationLocation is null or empty");
        }


        logger.debug("initializing authenticationBindings");
        logger.debug("authenticationBindingsImpl=" + authenticationBindingsFactoryImpl);
        logger.debug("filterConfigurationLocation=" + filterConfigurationLocation);
        AuthenticationBindingsFactory authNBindingsFactory = null;
        try {
            Class authenticationBindingsFactoryClass = Thread.currentThread().getContextClassLoader().loadClass(authenticationBindingsFactoryImpl);
            Class[] constructorArgumentTypes = new Class[]{String.class};
            Constructor constructor = authenticationBindingsFactoryClass.getConstructor(constructorArgumentTypes);
            authNBindingsFactory = (AuthenticationBindingsFactory) constructor.newInstance(authenticationScope);
            authNBindingsFactory.init(filterConfigurationLocation);

        } catch (InstantiationException iex) {
            logger.error(iex.getMessage(), iex);
            throw new IllegalArgumentException(iex);
        } catch (ClassNotFoundException cne) {
            logger.error(cne.getMessage(), cne);
            throw new IllegalArgumentException(cne);
        } catch (IllegalAccessException iae) {
            logger.error(iae.getMessage(), iae);
            throw new IllegalArgumentException(iae);
        } catch (SecurityException e) {
            logger.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        } catch (NoSuchMethodException e) {
            logger.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        } catch (IllegalArgumentException e) {
            logger.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        } catch (InvocationTargetException e) {
            logger.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        }
        return authNBindingsFactory;
    }

    /**
     * authenticate client request after registering it.
     *
     * @param authenticationBindings
     * @return <code>true</code> if authentication succeeds, <code>false</code> otherwise
     * @throws net.sf.jguard.core.authentication.AuthenticationException when authentication fails
     *
     */
    private AuthenticationStatus authenticateAfterRegistration(AuthenticationBindings authenticationBindings) throws AuthenticationException {
        authenticationBindings.setRequestAttribute(CoreConstants.REGISTRATION_DONE, Boolean.TRUE);
        return AuthenticationServicePoint.authenticate(authenticationBindings, applicationName, authenticationScope).getStatus();

    }


    /**
     * we remove the AuthenticationUtils (wrapper around Subject) and invalidate the session.
     * only useful for StatefulAuthenticationBindings AuthenticationBindings.
     *
     * @param authenticationBindings
     * @see StatefulAuthenticationBindings
     * @see AuthenticationBindings
     */
    private void logoff(AuthenticationBindings authenticationBindings) {
        logger.debug(" logoff phase ");
        StatefulAuthenticationBindings statefulAuthenticationBindings = (StatefulAuthenticationBindings) authenticationBindings;
        //remove Subject from session
        AuthenticationUtils auth = (net.sf.jguard.core.authentication.AuthenticationUtils) statefulAuthenticationBindings.getSessionAttribute(CoreConstants.AUTHN_UTILS);
        if (auth != null) {
            auth.logout();
            logger.debug(" user logoff ");
        }

        statefulAuthenticationBindings.removeSessionAttribute(net.sf.jguard.core.CoreConstants.AUTHN_UTILS);

        logger.debug("doFilter() -  user logoff ");

        //we invalidate the session to unbound all objects, including subject
        try {
            statefulAuthenticationBindings.invalidateSession();
        } catch (java.lang.IllegalStateException ise) {
            logger.error(" session is already invalidated ", ise);
        }
    }


    /**
     * propagate the call with security information into the Thread, i.e
     * under the hood in the JVM.
     *
     * @param context map of parameters
     * @param subject authenticated user
     * @param chain   filterChain to propagate to the next filter the call with a securized call
     */
    private void propagateWithSecurity(AccessContext context,final Subject subject, FilterChain chain) {
        final AccessContext accContext = context;
        final FilterChain c = chain;
        
        //propagate security information into the Thread
        try {
            Subject.doAsPrivileged(subject, new PrivilegedExceptionAction() {
                public Object run() throws IOException {
                    //we wrap the ServletRequest to 'correct' the j2ee's JAAS handling
                    // according to the j2se way
                    try {
                        policyDecisionPoint.process(accContext, c);
                    } catch (Throwable t) {
                        logger.error(t.getMessage(), t);
                        throw new RuntimeException(t.getMessage(), t);
                    }
                    // the 'null' tells the SecurityManager to consider this resource access
                    //in an isolated context, ignoring the permissions of code currently
                    //on the execution stack.
                    return null;
                }
            }, null);
        } catch (Throwable t) {
            logger.error(t.getMessage(), t);
            throw new RuntimeException(t.getMessage(), t);
        }
    }


    /**
     * does access to this resource (envisioned here as a permission) requires authentication ?
     * i.e, does GUEST has got access to this resource?
     *
     * @param authBindings authenticationBindings specific to the underlying technology
     * @param permission   permission the user tries to enforce
     * @return true if access is granted to guest, false otherwise
     */
    private boolean accessGrantedToGuest(AuthenticationBindings authBindings, Permission permission) {
        Subject guest;
        try {
            guest = getGuestSubject(authBindings);
        } catch (AuthenticationException ex) {
            logger.error(ex.getMessage(), ex);
            return false;
        }
        return AccessControllerUtils.hasPermission(guest, permission);
    }

    /**
     * return <b>true</b> if user answer to an authentication challenge,
     * <b>false</b> if access to 'guest' is not granted,
     * <b>true</b> if no authenticated user is present,
     * <b>false</b> otherwise.
     *
     * @param authBindings authenticationBindings specific to the underlying technology
     * @param permission   permission the user tries to enforce
     * @return true if authentication is needed, false otherwise
     */
    private boolean checkAuthenticationNeeded(AuthenticationBindings authBindings, Permission permission) {
        //user request to authenticate
        if (AuthenticationServicePoint.answerToChallenge(authBindings.getContext(), authenticationBindingsFactory)) {
            return true;
        }

        //authentication is nos requested to user if access is granted to 'Guest'
        if (accessGrantedToGuest(authBindings, permission)) {
            return false;
        }

        //if permission is not granted to guest and if there is no authenticated user
        //an authentication challenge is needed:
        // it will be fired with the authentication process via the callbackhandler
        Subject currentSubject = getCurrentSubject(authBindings);
        return currentSubject == null;

    }

    /**
     * filter call in a secure way.
     * note that this method can throw {@link Exception} or {@link Error}.
     *
     * @param context map of parameters
     * @param chain invocation Filter chain
     */
    private void doInternalFilter(AccessContext context, FilterChain chain) {
        AuthenticationBindings authenticationBindings = authenticationBindingsFactory.getAuthenticationBindings(context);
        Permission permission = policyDecisionPoint.getAuthorizationBindings().getPermissionRequested(authenticationBindings.getContext());

        boolean authenticationNeeded = checkAuthenticationNeeded(authenticationBindings, permission);
        Subject subject;

        try {
            subject = getCurrentSubject(authenticationBindings);
            if (subject == null) {
                AuthenticationUtils guestAuthnUtils = AuthenticationServicePoint.impersonateAsGuest(authenticationBindings, applicationName, authenticationScope);
                subject = guestAuthnUtils.getSubject();
            }
        } catch (AuthenticationException ex) {
            logger.error("an authentication exception occurs" + ex.getMessage());
            throw new RuntimeException(ex);
        }

        if (authenticationNeeded) {
            //authentication is needed
            try {
                //user is not authenticated and authentication is required
                AuthenticationStatus authStatus = AuthenticationServicePoint.authenticate(authenticationBindings, applicationName, authenticationScope).getStatus();
                if (!AuthenticationStatus.SUCCESS.equals(authStatus)) {
                    //authentication continue with another roundtrip
                    //or authentication failed
                    return;
                } else {
                    //authentication SUCCEED
                    //we regrab the subject (i.e the wrapping authenticationUtils) from the context)
                    //because we've authenticated the subject successfully
                    subject = getCurrentSubject(authenticationBindings);
                }

            } catch (AuthenticationException ex) {
                logger.error(" authentication of guest raise an exception", ex);
                throw new RuntimeException(ex);
            }

            //only if the authentication has been done with StatefulAuthenticationSchemeHandler
        } else if (userTriesToLogoff(subject, permission, authenticationBindingsFactory)) {
            logoff(authenticationBindings);
            try {
                AuthenticationServicePoint.impersonateAsGuest(authenticationBindings, applicationName, authenticationScope);
            } catch (AuthenticationException ex) {
                logger.error("authentication as guest fails during logoff phase" + ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }

        }

        authorize(authenticationBindings, subject, chain);

    }

    /**
     * checks user has been authenticated with a <b>StatefulAuthenticationBindings</b>AuthenticationSchemeHandler (only those
     * has got the logoff feature which is meanless in other cases), and check that the permission requested
     * is implied by the logoff permission of the StatefulAuthenticationHandler.
     *
     * @param subject current user
     * @param permissionToCheck permission enforced
     * @param authenticationBindingsFactory factory of authenticationBindings
     * @return <i>true</i> if user tries to logoff, <i>false</i> otherwise
     */
    private boolean userTriesToLogoff(Subject subject, Permission permissionToCheck, AuthenticationBindingsFactory authenticationBindingsFactory) {
        boolean userLogoff = false;

        //we check that authenticationSchemeHandler is stateful and if logoff is called
        AuthenticationSchemeHandler authSchemeHandler = AuthenticationServicePoint.getAuthenticationSchemeHandler(subject, authenticationBindingsFactory);
        if (authSchemeHandler == null) {
            //when user is guest which use impersonationAUthenitcationBindings not present here
            return false;
        }
        if (StatefulAuthenticationSchemeHandler.class.isAssignableFrom(authSchemeHandler.getClass())) {
            StatefulAuthenticationSchemeHandler statefulAuthSchemeHandler = (StatefulAuthenticationSchemeHandler) authSchemeHandler;
            if (statefulAuthSchemeHandler.getLogoffPermission().implies(permissionToCheck)) {
                userLogoff = true;
            }
        }

        return userLogoff;
    }

    /**
     * filter call to securize it, i.e authenticate the user which raises it, check authorizations
     * on it, and apply some checks on confidentiality and integrity.
     * Filtering is done in a <i>safe</i> way, i.e without raising any {@link Throwable}.
     *
     * @param context map of parameters
     * @param chain   handle Filters invocation successively
     */
    public void doFilter(AccessContext context, FilterChain chain) {
        try {
            doInternalFilter(context, chain);
        } catch (Throwable t) {
            logger.error(t.getMessage());
            //for debug purpose, we throw the exception
            //MUST only used in DEVELOPMENT phase
            //NEVER in PRODUCTION phase
            if (propagateThrowable) {
                throw new IllegalStateException(t);
            } else {
                getPolicyDecisionPoint().getAuthorizationBindings().sendThrowable(context, t);
            }
        }
    }

}
